import { Canvas, Meta } from "@storybook/addon-docs";

<Meta title="Components/Stateful Filter & Content/Stateful Content/Documentation" />

# Stateful Content

The Stateful Content API provides a standard for handling filtered list data in frontend components,
when content data and filter options are fetched from REST API endpoints.

## Content DataProvider

The `ContentDataProvider` can be used to retrieve data delivered by the Mytrumpf content REST API.

Calling the `getContentDataProvider` function with a specific scope creates a new instance if it does not yet exist
and returns the scoped `ContentDataProvider` instance.

Any component can subscribe to `ContentDataProvider` state changes.

### Filter Binding

The `ContentDataProvider` itself subscribes to the related `FilterDataProvider` with the same `scope`.

A `ContentDataProvider` subscriber automatically receives the updated, filtered data whenever the filter was changed.

**See [Stateful Filter Docs](./?path=/docs/components-stateful-content-stateful-filter-documentation--page)**

### Persistence

Whenever `query` or `sort` have changed, the values are stored in the `window.sessionStore`
with the key `trumpf.ux.iws.statefulContent.dataProvider.{SCOPE}`.

When a provider is instantiated, it reads the last stored values for `query` and `sort` and
updates the local state accordingly.

### Usage

```typescript
import {
    ContentDataProvider,
    ContentDataProviderState,
    getContentDataProvider,
} from "../../apis/stateful-content";

// get content data provider instance for specific scope
const dataProvider: ContentDataProvider = getContentDataProvider("myscope");

// set api endpoint
dataProvider.setEndpoint("/api/content/mysoftware");

// set query parameters
dataProvider.setQuery("foo=bar&lorem=ipsum");

// set sort query parameter
dataProvider.setSortParam("name-asc");

// subscribe
dataProvider.subscribe(({ data, error, pending }: ContentDataProviderState) => {
    ...
});
```

### ContentDataProvider State

See [Content Data Interface](#content-data-interface) for details.

```typescript
export interface ContentDataProviderState<T = any> {
    /** The last api request data */
    readonly data: ContentData<T> | null;
    /** Api endpoint */
    readonly endpoint: string | null;
    /** The last api request error */
    readonly error: ContentApiError | null;
    /** Flag that indicates if a request is pending */
    readonly pending: boolean;
    /** Query params for the api request */
    readonly query: string;
    /** Sort query param */
    readonly sort: string;
}
```

## Content API Endpoint

REST API Endpoint for stateful content.

-   Accepts filter query parameters as defined in the according filter endpoint
-   Accepts sort query parameter
-   Provides array with list items, result summary or error message
-   Provides a schema which describes the data

### Content Data Interface

```typescript
export interface ContentData<T = any> {
    /** number of filtered results */
    filteredCount: number;
    /** number of total results */
    totalCount: number;
    /** array with result items */
    items: Array<T>;
    /** schema used to render the according view */
    schema?: ContentDataSchema;
    /** optional error */
    error?: ContentApiError;
    /** @deprecated Label used for displaying result summary. Use translations instead. */
    labelCount?: string;
}

export interface ContentDataSchema {
    /** Label for the data field */
    [field: string]: ContentDataSchemaField;
}

export interface ContentDataSchemaField extends Record<string, any> {
    /** Label for the data field */
    label?: string;
    /** @deprecated Set of labels for the field if used without translations */
    labels?: Record<string, string>;
    /** Flag that indicates if data can be sorted by this field */
    sortable?: boolean;
}

export interface ContentApiError {
    /** Status code of the response */
    status?: number;
    /** Error message */
    message?: string;
    /** Optional error headline */
    head?: string;
    /** Optional error copy text */
    copy?: string;
}
```

#### Content API Error

```typescript
export interface ContentApiError {
    /** Status code of the response */
    status?: number;
    /** Error message */
    message?: string;
    /** Optional error headline */
    head?: string;
    /** Optional error copy text */
    copy?: string;
}
```

### Example response data

```json
{
    "totalCount": 100,
    "filteredCount": 3,
    "schema": {
        "category": {
            "order": [1001, 1002, 1003, 1004]
        },
        "date": {
            "label": "Release Date"
        },
        "languages": {
            "label": "Languages"
        },
        "file": {
            "label": "Download"
        }
    },
    "items": [
        {
            "name": "Operation Manual PFO 3D-2",
            "languages": ["en"],
            "date": "2017-06-01",
            "file": "https://my.trumpf.com/api/download?id=1142603",
            "size": 4725399,
            "category": 1001
        },
        {
            "name": "Operation Manual PFO 3D-2",
            "languages": ["en"],
            "date": "2019-07-01",
            "file": "https://my.trumpf.com/api/download?id=1142797",
            "size": 5916769,
            "category": 1001
        },
        {
            "name": "Spare Parts Catalog PFO 3D-2, PFO 3D-2 PO, I-PFO",
            "languages": ["de", "fr", "en"],
            "date": "2021-11-01",
            "file": "https://my.trumpf.com/api/download?id=1153222",
            "size": 3928555,
            "category": 1004
        }
    ]
}
```

## Content Translations (API V2)

Some `<ux-iws-stateful-*>` content components have a `translations` prop which can be used to pass in
static translations as JSON string or object.

If static translations are used, localized texts can be omitted in the API response.

### Supported Components

-   \*\*[ux-iws-stateful-download](./?path=/docs/components-stateful-content-stateful-content-stateful-downloads--stateful-downloads)

### Example Translations

```json
{
    "category": {
        "values": {
            "1001": "Betriebsanleitungen",
            "1004": "Software-Handbücher & Programmieranleitungen"
        }
    },
    "date": {
        "label": "Ausgabedatum"
    },
    "language": {
        "label": "Sprache",
        "values": {
            "en": "English",
            "de": "Deutsch",
            "fr": "Français",
            "it": "Italiano"
        }
    }
}
```

### Example response data without localized text

If used with translations, the `label` fields can be omitted in the `schema` of the API response:

```json
{
    "totalCount": 0,
    "filteredCount": 0,
    "schema": {
        "category": {
            "order": [1001, 1002, 1003, 1004]
        },
        "date": {},
        "languages": {},
        "file": {}
    },
    "items": []
}
```
